package org.act.od.impl.viewhelper {
import com.adobe.cairngorm.view.ViewHelper;

import flash.events.MouseEvent;

import mx.collections.ArrayCollection;
import mx.core.Application;
import mx.events.CloseEvent;
import mx.managers.PopUpManager;

import org.act.od.impl.view.toolMenuBar.DesignerToolBar;
import org.ow2.orchestra.designer.events.FileNavigatorEvent;
import org.ow2.orchestra.designer.events.ListUserOfDesignerEvent;
import org.ow2.orchestra.designer.events.ProcessEditorEvent;
import org.ow2.orchestra.designer.events.UndoRedoEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.utils.ErrorManager;
import org.ow2.orchestra.designer.view.CreateNewFileWindow;
import org.ow2.orchestra.designer.view.accessProperties.AccessPropertyView;
import org.ow2.orchestra.designer.vo.FileNavigatorItem;
import org.ow2.orchestra.designer.vo.FileNavigatorOwner;
import org.ow2.orchestra.designer.vo.FileNavigatorProcess;
import org.ow2.orchestra.designer.vo.FileNavigatorProject;

/**
 * The ViewHelper of DesignerToolBar.
 * Used to isolate command classes from the implementation details of a view.
 */
public class DesignerToolBarVH extends ViewHelper {
  /**
   * The key of DesignerToolBarVH
   */
  public static const VH_KEY:String = "DesignerToolBarVH";

  /**
   * Initialize DesignerToolBar with DesignerToolBarVH
   */
  public function DesignerToolBarVH(document:Object, id:String) {
    super();
    initialized(document, id);
  }

  /**
   * Return DesignerToolBar.
   */
  private function get designerToolBar():DesignerToolBar {
    return this.view as DesignerToolBar;
  }

  /**
   * The handler of "New Project" button.
   */
  public function newProjectHandler(event:MouseEvent):void {
    var newProject:CreateNewFileWindow = CreateNewFileWindow(PopUpManager.createPopUp(Application.application as Application, CreateNewFileWindow, true));
    newProject.setTitle("Project");
    PopUpManager.centerPopUp(newProject);
    newProject.addEventListener(CloseEvent.CLOSE, newProjectResult);
  }

  private function newProjectResult(event:CloseEvent):void {
    var projectName:String = CreateNewFileWindow(event.currentTarget).getText();
    if (projectName != "") {
      var owner:FileNavigatorOwner = new FileNavigatorOwner();
      owner.name = DesignerModelLocator.getInstance().userName;
      var project:FileNavigatorProject = new FileNavigatorProject();
      project.name = projectName;
      project.owner = owner;
      owner.projects.addItem(project);
      new FileNavigatorEvent(FileNavigatorEvent.PROJECT_CREATE, project).dispatch();
    }
  }

  /**
   * The handler of "New File" button.
   */
  public function newFileHandler(event:MouseEvent):void {
    var fileNavigatorSelectedItem:FileNavigatorItem =
        DesignerModelLocator.getInstance().fileNavigatorModel.selectedItem;
    if (fileNavigatorSelectedItem == null) {
      ErrorManager.getInstance().showError("Please select a project first.");
    } else {
      var newFile:CreateNewFileWindow =
          CreateNewFileWindow(PopUpManager.createPopUp(Application.application as Application, CreateNewFileWindow, true));
      newFile.setTitle("Process");
      PopUpManager.centerPopUp(newFile);
      newFile.addEventListener(CloseEvent.CLOSE, newFileResult);
    }
  }

  private function newFileResult(event:CloseEvent):void {
    var fileName:String = CreateNewFileWindow(event.currentTarget).getText();
    if (fileName != "") {
      var process:FileNavigatorProcess = new FileNavigatorProcess();
      process.name = fileName;
      var fileNavigatorSelectedItem:FileNavigatorItem = DesignerModelLocator.getInstance().fileNavigatorModel.selectedItem;
      if (fileNavigatorSelectedItem is FileNavigatorProject) {
        process.project = fileNavigatorSelectedItem as FileNavigatorProject;
      }
      else if (fileNavigatorSelectedItem is FileNavigatorProcess) {
        process.project = (fileNavigatorSelectedItem as FileNavigatorProcess).project;
      }
      new FileNavigatorEvent(FileNavigatorEvent.PROCESS_CREATE, process).dispatch();
    }
  }

  /**
   * The handler of "Save" button.
   */
  public function onSaveHandler(event:MouseEvent):void {
    var models:ArrayCollection = new ArrayCollection();
    var selectedProcess:ProcessEditorModel = DesignerModelLocator.getInstance().selectedProcessEditor;
    if (selectedProcess != null)
      models.addItem(selectedProcess);
    new ProcessEditorEvent(ProcessEditorEvent.SAVE_MODELS, models).dispatch();
  }

  /**
   * The handler of "Export" button.
   */
  public function onExportHandler(event:MouseEvent):void {
    var models:ArrayCollection = new ArrayCollection();
    var selectedProcess:ProcessEditorModel = DesignerModelLocator.getInstance().selectedProcessEditor;
    if (selectedProcess != null)
      models.addItem(selectedProcess);
    new ProcessEditorEvent(ProcessEditorEvent.EXPORT_MODELS, models).dispatch();
  }

  /**
   * The handler of "Deploy" button.
   */
  public function onDeployHandler(event:MouseEvent):void {
    var models:ArrayCollection = new ArrayCollection();
    var selectedProcess:ProcessEditorModel = DesignerModelLocator.getInstance().selectedProcessEditor;
    if (selectedProcess != null)
      models.addItem(selectedProcess);
    new ProcessEditorEvent(ProcessEditorEvent.DEPLOY_MODELS, models).dispatch();
  }

  /**
   * The handler of "Save All" button.
   */
  public function onSaveAllHandler(event:MouseEvent):void {
    var models:ArrayCollection = new ArrayCollection();
    models.addAll(DesignerModelLocator.getInstance().openedProcesses);
    new ProcessEditorEvent(ProcessEditorEvent.SAVE_MODELS, models).dispatch();
  }

  /**
   * The handler of "Cut" button.
   */
  public function onCutHandler(event:MouseEvent):void {
  }

  /**
   * The handler of "Copy" button.
   */
  public function onCopyHandler(event:MouseEvent):void {
  }

  /**
   * The handler of "Paste" button.
   */
  public function onPasteHandler(event:MouseEvent):void {
  }

  /**
   * The handler of "Zoom In" button.
   */
  public function onZoomInHandler(event:MouseEvent):void {
  }

  /**
   * The handler of "Zoom Out" button.
   */
  public function onZoomOutHandler(event:MouseEvent):void {
  }

  public function onUndo(event:MouseEvent):void {
    new UndoRedoEvent(UndoRedoEvent.UNDO).dispatch();
  }

  public function onRedo(event:MouseEvent):void {
    new UndoRedoEvent(UndoRedoEvent.REDO).dispatch();
  }

  /**
   * The handler of "onProcessAccessConfigButton" button.
   */
  public function onProcessAccessConfigHandler(event:MouseEvent):void {
    new ListUserOfDesignerEvent(ListUserOfDesignerEvent.LIST_USERS_OF_DESIGNER).dispatch();
    var orchestraSettings:AccessPropertyView = AccessPropertyView(PopUpManager.createPopUp(Application
        .application as Application, AccessPropertyView, true));
    PopUpManager.centerPopUp(orchestraSettings);
  }
}
}

