/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.business {
import com.adobe.cairngorm.CairngormError;
import com.adobe.cairngorm.CairngormMessageCodes;

import flash.events.TimerEvent;
import flash.utils.Timer;

import org.ow2.orchestra.designer.events.PingServerEvent;

/**
 * @author Loic Albertin
 */
public class PingService {
  private static var _instance:PingService = null;

  private var _timer:Timer;

  // timeout not lower 200ms , by default 30000ms
  private static const initialTimeout:Number = 30000;
  private static const minimumTimeout:Number = 200;

  public function PingService() {
    if (_instance != null) {
      throw new CairngormError(CairngormMessageCodes.SINGLETON_EXCEPTION, "Singleton instance already exists." +
          " Do not call constructor directly, prefer getInstance method");
    }
  }

  public static function getInstance():PingService {
    if (_instance == null) {
      _instance = new PingService();
    }
    return _instance;
  }


  public function start():void {
    //Run infinitely
    if (_timer != null) {
      _timer = new Timer(initialTimeout, 0);
      _timer.addEventListener(TimerEvent.TIMER, timerHandler);
      _timer.start();
    }
  }

  public function stop():void {
    if (_timer != null) {
      _timer.stop();
      _timer.removeEventListener(TimerEvent.TIMER, timerHandler);
      _timer = null;
    }
  }

  public function restoreInitialDelay():void {
    if (_timer != null && _timer.delay != initialTimeout) {
      _timer.delay = initialTimeout;
    }
  }

  public function reduceDelay():void {
    if (_timer != null && _timer.delay > minimumTimeout) {
      _timer.delay /= 2;
      if (_timer.delay < minimumTimeout) {
        _timer.delay = minimumTimeout;
      }
    }
  }

  public function timerHandler(event:TimerEvent):void {
    // throw the refresh of tokens
    new PingServerEvent(PingServerEvent.PING_SERVER).dispatch();
  }
}
}
