/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands {
import com.adobe.cairngorm.business.ServiceLocator;
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;

import mx.collections.ArrayCollection;
import mx.managers.CursorManager;
import mx.rpc.AsyncToken;
import mx.rpc.Responder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
import mx.rpc.remoting.RemoteObject;

import org.ow2.orchestra.designer.bpmn.model.ProcessModel;
import org.ow2.orchestra.designer.events.ProcessEditorEvent;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.utils.ErrorManager;
import org.ow2.orchestra.designer.vo.FileNavigatorProcess;

/**
 * @author Loic Albertin
 */
public class ModelSaveCMD implements ICommand {
  public function ModelSaveCMD() {
  }

  protected var processEditorModelList:ArrayCollection;
  protected var currentIndex:int = -1;

  public function execute(event:CairngormEvent):void {
    if (event is ProcessEditorEvent) {
      var pee:ProcessEditorEvent = event as ProcessEditorEvent;
      // load processmodel list if the Save model event is triggered.
      // if the event is not a save event (export event), a subclass can load the list externally.
      if (pee.type == ProcessEditorEvent.SAVE_MODELS) {
        processEditorModelList = pee.processEditorModelList;
      }
      // set busy cursor and start saving the list of processes
      CursorManager.setBusyCursor();
      this.saveNextProcessEditor();
    }
  }

  private function saveNextProcessEditor():void {
    currentIndex++;
    if (currentIndex < processEditorModelList.length) {
      if ((processEditorModelList.getItemAt(currentIndex) as ProcessEditorModel).readWrite) {
        //processEditor = processEditorModelList.getItemAt(currentIndex) as ProcessEditorModel
        this.saveProcessEditor(processEditorModelList.getItemAt(currentIndex) as ProcessEditorModel);
      } else {
        saveNextProcessEditor();
      }
    } else {
      CursorManager.removeBusyCursor();
    }
  }

  private function saveProcessEditor(processEditor:ProcessEditorModel):void {
    if (processEditor.readWrite) {
      var process:FileNavigatorProcess = processEditor.fileNavigatorProcess;
      var processModel:ProcessModel = processEditor.processModel;
      var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("fileNavigator");

      var token:AsyncToken = remote.saveProcess(process, processModel);
      token.addResponder(new Responder(saveModelResult, fault));
    }
  }

  /**
   * When save is finished, start saving next process.
   * This method is protected to be overriden by subclasses (export will export when save is finished, before qving the next process).
   */
  protected function saveModelResult(event:ResultEvent = null):void {
    saveNextProcessEditor();
  }

  /**
   * Function used in fault. Subclass override this method to have the right comment in the popup window.
   */
  protected function get commandName():String {
    return "save";
  }

  protected function fault(event:FaultEvent):void {
    CursorManager.removeBusyCursor();
    ErrorManager.getInstance().changeLevelInfo(0);
    ErrorManager.getInstance().updateTitle("Error : read only process")
    if (event.fault.faultString != null && event.fault.faultString != "") {
      if (event.fault.faultString.search("Permission denied") != -1) {
        ErrorManager.getInstance().updateTitle("Error : read only process");
      }
      ErrorManager.getInstance().showError("Error during " + commandName + ": " + event.fault.faultString);
    } else {
      ErrorManager.getInstance().showError("Error during " + commandName + ". Please look at the server logs for more " +
          "information.");
    }
  }
}
}
